import os
import logging
from .json_formatter import formatter as json_formatter, JSON_FORMATTER_CONFIG
from .string_formatter import formatter as string_formatter, StringFormatter, STRING_FORMATTER_CONFIG

is_local = os.getenv('ENV') in ['local', None, '']

logger = logging.getLogger()
logger.setLevel(logging.DEBUG if os.getenv('LOG_DEBUG') else logging.INFO)

log_handler = logging.StreamHandler()

if is_local:
    original_makeRecord = logging.Logger.makeRecord

    def make_record_with_extra(self, name, level, fn, lno, msg, args, exc_info, func=None, extra=None, sinfo=None):
        record = original_makeRecord(
            self, name, level, fn, lno, msg, args, exc_info, func, extra, sinfo)
        record._extra = extra
        return record

    logging.Logger.makeRecord = make_record_with_extra

    log_handler.setFormatter(string_formatter)
else:
    log_handler.setFormatter(json_formatter)

logger.addHandler(log_handler)

FORMATTER_CONFIG = STRING_FORMATTER_CONFIG if is_local else JSON_FORMATTER_CONFIG

__all__ = ['logger', 'StringFormatter', 'FORMATTER_CONFIG']



import os
import socket
import traceback
from pythonjsonlogger.json import JsonFormatter
from pythonjsonlogger.core import RESERVED_ATTRS

JSON_FORMATTER_CONFIG = {
    'fmt': '{message}{asctime}{levelno}{exc_info}',
    'style': '{',
    'datefmt': '%Y-%m-%dT%H:%M:%S%z',
    '()': 'pythonjsonlogger.jsonlogger.JsonFormatter',
    'rename_fields': {
        'asctime': 'timestamp',
        'message': 'msg',
        'levelno': 'level',
        'exc_info': 'exception'
    },
    'reserved_attrs': RESERVED_ATTRS+['color_message', 'exc_info'],
    'static_fields': {
        'name': os.getenv('APP_NAME'),
        'pid': os.getpid(),
        'hostname': socket.gethostname()
    }
}


class CustomJsonFormatter(JsonFormatter):
    def add_fields(self, log_record, record, message_dict):
        super(CustomJsonFormatter, self).add_fields(
            log_record, record, message_dict)

        if record.exc_info:
            log_record['exception'] = {
                'type': record.exc_info[0].__name__,
                'message': str(record.exc_info[1]),
                'traceback': traceback.format_exception(*record.exc_info)
            }


formatter = CustomJsonFormatter(
    JSON_FORMATTER_CONFIG['fmt'],
    style=JSON_FORMATTER_CONFIG['style'],
    static_fields=JSON_FORMATTER_CONFIG['static_fields'],
    rename_fields=JSON_FORMATTER_CONFIG['rename_fields'],
    reserved_attrs=JSON_FORMATTER_CONFIG['reserved_attrs'],
    datefmt=JSON_FORMATTER_CONFIG['datefmt']
)



import json
import logging
import traceback
from copy import copy

STRING_FORMATTER_CONFIG = {
    'fmt': '{asctime} {levelprefix}  {message}{extra}{exception}',
    'style': '{',
    'datefmt': '%Y-%m-%dT%H:%M:%S%z',
    '()': 'defix_package_logger.StringFormatter'
}


class StringFormatter(logging.Formatter):
    def formatMessage(self, record: logging.LogRecord) -> str:
        recordcopy = copy(record)
        levelname = recordcopy.levelname
        recordcopy.__dict__["levelprefix"] = levelname + ":"

        extra_obj = recordcopy._extra
        if extra_obj:
            recordcopy.__dict__['extra'] = '\n' + \
                json.dumps(extra_obj, indent=2)
        else:
            recordcopy.__dict__['extra'] = ''

        if recordcopy.exc_info:
            exception_info = {
                'type': recordcopy.exc_info[0].__name__,
                'message': str(recordcopy.exc_info[1]),
                'traceback': ''.join(traceback.format_exception(*recordcopy.exc_info))
            }
            recordcopy.__dict__['exception'] = '\nException:\n' + \
                json.dumps(exception_info, indent=2)
        else:
            recordcopy.__dict__['exception'] = ''

        return super().formatMessage(recordcopy)

    def formatException(self, ei):
        return ""


formatter = StringFormatter(
    fmt=STRING_FORMATTER_CONFIG['fmt'],
    style=STRING_FORMATTER_CONFIG['style'],
    datefmt=STRING_FORMATTER_CONFIG['datefmt']
)



from defix_package_logger import logger


def main():
    logger.error('Logging', exc_info=ValueError('val'), extra={
        'msg_code': 'CRITICAL_ERROR'
    } | {'xid': '-', 'custom': 'cus', 'custom': {'22': '12'}})


if __name__ == "__main__":
    main()
